@@ -1,7 +1,10 @@
 import re
 from .CONSTANTS import STYLE, TXT, BG
 
-def expand_special_values(u_str): # Match Special Values
+def expand_rainbow(u_str): # Match Special Values
+    operationsRe='('+'|'.join(list(STYLE)+list(TXT)+list(BG))+')' #regex or statement of all the special keywords
+    parserRe=r'\['+operationsRe+r'\]'
+
     ## Rainbow
     rainbow_chars=[TXT['red'], TXT['orange'], TXT['yellow'], TXT['green'], TXT['blue'], TXT['lightblue'], TXT['purple']]
     global rainbow_char_i
@@ -13,15 +16,18 @@ def expand_special_values(u_str): # Match Special Values
         rainbow_char_i=(rainbow_char_i+1) % len(rainbow_chars)
         return out
 
-    matched=re.search(r'\[rainbow\]([^\\\[]*)', u_str) #matches all text except \ and [
-    if matched:
-        out=''
-        for char in matched.groups()[0]: #group of characters
-            out+=next_rainbow_char()+char
-        u_str=u_str[:matched.start()]+out+u_str[matched.end()-1:]
+    matchingRe=fr'(.*?)\[rainbow\](.*?)({parserRe}?.*)'
+    matched=re.search(matchingRe, u_str, re.DOTALL) #re.DOTALL allows multiline matching
+    if matched: #recursive step (expand all rainbows)
+        [starting, txt_to_rainbowfy, ending, operation]=matched.groups()
+        rainbowfied_txt='' #product
+        for char in txt_to_rainbowfy: #group of characters
+            rainbowfied_txt+=next_rainbow_char()+char
+
+        return starting+rainbowfied_txt+expand_rainbow(ending) #recursive step
+    else: #base case
+        return u_str
 
-    return u_str
-    
 def expand_style_txt_bg(u_str): # Match STYLE, TXT, BG
     operationsRe='('+'|'.join(list(STYLE)+list(TXT)+list(BG))+')' #regex or statement of all the special keywords
     parserRe=r'\['+operationsRe+r'\]'
@@ -49,6 +55,6 @@ def expand_style_txt_bg(u_str): # Match STYLE, TXT, BG
 
 
 def format(u_str): #returns with escaped color characters
-    u_str=expand_special_values(u_str)
+    u_str=expand_rainbow(u_str)
     u_str=expand_style_txt_bg(u_str)
     return u_str
